@@ -29,8 +29,11 @@
 #include "eel-lib-self-check-functions.h"
 #include <math.h>
 
-const ArtIRect eel_art_irect_empty = { 0, 0, 0, 0 };
 const ArtDRect eel_art_drect_empty = { 0.0, 0.0, 0.0, 0.0 };
+const ArtIRect eel_art_irect_empty = { 0, 0, 0, 0 };
+const ArtPoint eel_art_point_zero = { 0, 0 };
+const EelArtIPoint eel_art_ipoint_max = { G_MAXINT, G_MAXINT };
+const EelArtIPoint eel_art_ipoint_min = { G_MININT, G_MININT };
 const EelArtIPoint eel_art_ipoint_zero = { 0, 0 };
 const EelDimensions eel_dimensions_empty = { 0, 0 };
 
@@ -187,6 +190,12 @@ eel_art_irect_align (ArtIRect container,
 	return aligned;
 }
 
+gboolean
+eel_art_irect_is_empty (ArtIRect rectangle)
+{
+	return art_irect_empty (&rectangle);
+}
+
 /**
  * eel_dimensions_are_empty:
  * 
@@ -403,6 +412,19 @@ eel_dimensions_equal (EelDimensions dimensions_a,
 		&& dimensions_a.height == dimensions_b.height;
 }
 
+EelDimensions
+eel_dimensions_clamp (EelDimensions dimensions,
+		      EelDimensions min,
+		      EelDimensions max)
+{
+	EelDimensions clamped;
+
+	clamped.width = CLAMP (dimensions.width, min.width, max.width);
+	clamped.height = CLAMP (dimensions.height, min.height, max.height);
+
+	return clamped;
+}
+
 EelArtIPoint
 eel_art_ipoint_assign (int x,
 		       int y)
@@ -423,11 +445,28 @@ eel_art_ipoint_equal (EelArtIPoint point_a,
 		&& point_a.y == point_b.y;
 }
 
+EelArtIPoint
+eel_art_ipoint_clamp (EelArtIPoint point,
+		      EelArtIPoint min,
+		      EelArtIPoint max)
+{
+	return eel_art_ipoint_assign (CLAMP (point.x, min.x, max.x),
+				      CLAMP (point.y, min.y, max.y));
+}
+
+EelArtIPoint
+eel_art_ipoint_offset_by (EelArtIPoint point,
+			  int x,
+			  int y)
+{
+	return eel_art_ipoint_assign (point.x + x, point.y + y);
+}
+
 #if !defined (EEL_OMIT_SELF_CHECK)
 
 static ArtIRect
-test_intersect (int a_x0, int a_y0, int a_x1, int a_y1,
-		int b_x0, int b_y0, int b_x1, int b_y1)
+test_irect_intersect (int a_x0, int a_y0, int a_x1, int a_y1,
+		      int b_x0, int b_y0, int b_x1, int b_y1)
 {
 	ArtIRect a;
 	ArtIRect b;
@@ -446,8 +485,8 @@ test_intersect (int a_x0, int a_y0, int a_x1, int a_y1,
 }
 
 static ArtIRect
-test_union (int a_x0, int a_y0, int a_x1, int a_y1,
-	    int b_x0, int b_y0, int b_x1, int b_y1)
+test_irect_union (int a_x0, int a_y0, int a_x1, int a_y1,
+		  int b_x0, int b_y0, int b_x1, int b_y1)
 {
 	ArtIRect a;
 	ArtIRect b;
@@ -465,6 +504,22 @@ test_union (int a_x0, int a_y0, int a_x1, int a_y1,
 	return eel_art_irect_union (a, b);
 }
 
+static EelArtIPoint
+test_ipoint_clamp (int x, int y, int min_x, int min_y, int max_x, int max_y)
+{
+	return eel_art_ipoint_clamp (eel_art_ipoint_assign (x, y),
+				     eel_art_ipoint_assign (min_x, min_y),
+				     eel_art_ipoint_assign (max_x, max_y));
+}
+
+static EelDimensions
+test_dimensions_clamp (int width, int height, int min_width, int min_height, int max_width, int max_height)
+{
+	return eel_dimensions_clamp (eel_dimensions_assign (width, height),
+				     eel_dimensions_assign (min_width, min_height),
+				     eel_dimensions_assign (max_width, max_height));
+}
+
 void
 eel_self_check_art_extensions (void)
 {
@@ -569,19 +624,37 @@ eel_self_check_art_extensions (void)
 	EEL_CHECK_POINT_RESULT (eel_art_ipoint_assign (0, -1), 0, -1);
 	EEL_CHECK_POINT_RESULT (eel_art_ipoint_assign (-1, 0), -1, 0);
 
-	EEL_CHECK_RECTANGLE_RESULT (test_intersect (0, 0, 0, 0, 0, 0, 0, 0), 0, 0, 0, 0);
-	EEL_CHECK_RECTANGLE_RESULT (test_intersect (-1, -1, -1, -1, -1, -1, -1, -1), 0, 0, 0, 0);
-	EEL_CHECK_RECTANGLE_RESULT (test_intersect (-2, -2, -2, -2, -2, -2, -2, -2), 0, 0, 0, 0);
-	EEL_CHECK_RECTANGLE_RESULT (test_intersect (0, 0, 10, 10, 0, 0, 0, 0), 0, 0, 0, 0);
-	EEL_CHECK_RECTANGLE_RESULT (test_intersect (0, 0, 10, 10, 0, 0, 10, 10), 0, 0, 10, 10);
-	EEL_CHECK_RECTANGLE_RESULT (test_intersect (0, 0, 10, 10, 0, 0, 5, 5), 0, 0, 5, 5);
-	EEL_CHECK_RECTANGLE_RESULT (test_intersect (-5, -5, 5, 5, 5, 5, 6, 6), 0, 0, 0, 0);
-	EEL_CHECK_RECTANGLE_RESULT (test_intersect (-5, -5, 5, 5, 4, 4, 6, 6), 4, 4, 5, 5);
-	EEL_CHECK_RECTANGLE_RESULT (test_intersect (10, 10, 100, 100, 10, 10, 11, 11), 10, 10, 11, 11);
+	/* test_irect_intersect */
+	EEL_CHECK_RECTANGLE_RESULT (test_irect_intersect (0, 0, 0, 0, 0, 0, 0, 0), 0, 0, 0, 0);
+	EEL_CHECK_RECTANGLE_RESULT (test_irect_intersect (-1, -1, -1, -1, -1, -1, -1, -1), 0, 0, 0, 0);
+	EEL_CHECK_RECTANGLE_RESULT (test_irect_intersect (-2, -2, -2, -2, -2, -2, -2, -2), 0, 0, 0, 0);
+	EEL_CHECK_RECTANGLE_RESULT (test_irect_intersect (0, 0, 10, 10, 0, 0, 0, 0), 0, 0, 0, 0);
+	EEL_CHECK_RECTANGLE_RESULT (test_irect_intersect (0, 0, 10, 10, 0, 0, 10, 10), 0, 0, 10, 10);
+	EEL_CHECK_RECTANGLE_RESULT (test_irect_intersect (0, 0, 10, 10, 0, 0, 5, 5), 0, 0, 5, 5);
+	EEL_CHECK_RECTANGLE_RESULT (test_irect_intersect (-5, -5, 5, 5, 5, 5, 6, 6), 0, 0, 0, 0);
+	EEL_CHECK_RECTANGLE_RESULT (test_irect_intersect (-5, -5, 5, 5, 4, 4, 6, 6), 4, 4, 5, 5);
+	EEL_CHECK_RECTANGLE_RESULT (test_irect_intersect (10, 10, 100, 100, 10, 10, 11, 11), 10, 10, 11, 11);
 
-	EEL_CHECK_RECTANGLE_RESULT (test_union (0, 0, 0, 0, 0, 0, 0, 0), 0, 0, 0, 0);
-	EEL_CHECK_RECTANGLE_RESULT (test_union (-1, -1, -1, -1, -1, -1, -1, -1), 0, 0, 0, 0);
-	EEL_CHECK_RECTANGLE_RESULT (test_union (-2, -2, -2, -2, -2, -2, -2, -2), 0, 0, 0, 0);
+	/* test_irect_union */
+	EEL_CHECK_RECTANGLE_RESULT (test_irect_union (0, 0, 0, 0, 0, 0, 0, 0), 0, 0, 0, 0);
+	EEL_CHECK_RECTANGLE_RESULT (test_irect_union (-1, -1, -1, -1, -1, -1, -1, -1), 0, 0, 0, 0);
+	EEL_CHECK_RECTANGLE_RESULT (test_irect_union (-2, -2, -2, -2, -2, -2, -2, -2), 0, 0, 0, 0);
+
+	/* test_ipoint_clamp */
+	EEL_CHECK_POINT_RESULT (test_ipoint_clamp (0, 0, 0, 0, 0, 0), 0, 0);
+	EEL_CHECK_POINT_RESULT (test_ipoint_clamp (5, 5, 0, 0, 10, 10), 5, 5);
+	EEL_CHECK_POINT_RESULT (test_ipoint_clamp (0, 0, 0, 0, 10, 10), 0, 0);
+	EEL_CHECK_POINT_RESULT (test_ipoint_clamp (10, 10, 0, 0, 10, 10), 10, 10);
+	EEL_CHECK_POINT_RESULT (test_ipoint_clamp (11, 11, 0, 0, 10, 10), 10, 10);
+	EEL_CHECK_POINT_RESULT (test_ipoint_clamp (-1, -1, 0, 0, 10, 10), 0, 0);
+
+	/* test_dimensions_clamp */
+	EEL_CHECK_DIMENSIONS_RESULT (test_dimensions_clamp (0, 0, 0, 0, 0, 0), 0, 0);
+	EEL_CHECK_DIMENSIONS_RESULT (test_dimensions_clamp (5, 5, 0, 0, 10, 10), 5, 5);
+	EEL_CHECK_DIMENSIONS_RESULT (test_dimensions_clamp (0, 0, 0, 0, 10, 10), 0, 0);
+	EEL_CHECK_DIMENSIONS_RESULT (test_dimensions_clamp (10, 10, 0, 0, 10, 10), 10, 10);
+	EEL_CHECK_DIMENSIONS_RESULT (test_dimensions_clamp (11, 11, 0, 0, 10, 10), 10, 10);
+	EEL_CHECK_DIMENSIONS_RESULT (test_dimensions_clamp (-1, -1, 0, 0, 10, 10), 0, 0);
 }
 
 #endif /* !EEL_OMIT_SELF_CHECK */
